@@ -5,6 +5,9 @@
 */
 
 #include <KAndroidExtras/JniArray>
+#include <KAndroidExtras/JniTypeTraits>
+#include <KAndroidExtras/JavaTypes>
+#include <KAndroidExtras/Uri>
 
 #include <QtTest/qtest.h>
 
@@ -14,6 +17,12 @@ class JniArrayTest : public QObject
 {
     Q_OBJECT
 private Q_SLOTS:
+    void testTypeTrait()
+    {
+        static_assert(!Jni::is_array<int>::value);
+        static_assert(Jni::is_array<Jni::Array<int>>::value);
+    }
+
     void testFromArray()
     {
 #ifndef Q_OS_ANDROID
@@ -27,6 +36,14 @@ private Q_SLOTS:
         const auto a2 = Jni::fromArray<QStringList>(array);
         QCOMPARE(a2.size(), 2);
         QCOMPARE(a2, QStringList({ QStringLiteral("ctor: 0"), QStringLiteral("ctor: 1") }));
+
+        JNIEnv::m_arrayLength = 4;
+        const auto a3 = Jni::fromArray<std::vector<int>>(array);
+        QCOMPARE(a3.size(), 4);
+
+        JNIEnv::m_arrayLength = 1;
+        const auto a4 = Jni::fromArray<QVector<QUrl>>(array);
+        QCOMPARE(a4.size(), 1);
 #endif
     }
 };
