@@ -7,6 +7,7 @@
 #include "applicationcontroller.h"
 #include "documentmanager.h"
 #include "favoritelocationmodel.h"
+#include "filehelper.h"
 #include "gpxexport.h"
 #include "healthcertificatemanager.h"
 #include "importexport.h"
@@ -19,6 +20,7 @@
 #include "tripgroupmanager.h"
 #include <itinerary_version_detailed.h>
 
+#include <kitinerary_version.h>
 #include <KItinerary/CreativeWork>
 #include <KItinerary/DocumentUtil>
 #include <KItinerary/ExtractorCapabilities>
@@ -27,6 +29,9 @@
 #include <KItinerary/File>
 #include <KItinerary/JsonLdDocument>
 
+#include <KPkPass/Pass>
+
+#include <KAboutData>
 #include <KLocalizedString>
 
 #include <QBuffer>
@@ -48,10 +53,12 @@
 #include <QUrl>
 #include <QUrlQuery>
 
-#ifdef Q_OS_ANDROID
 #include <KMime/Message>
 #include <KMime/Types>
 
+#ifdef Q_OS_ANDROID
+#include "android/itineraryactivity.h"
+
 #include <kandroidextras/activity.h>
 #include <kandroidextras/contentresolver.h>
 #include <kandroidextras/intent.h>
@@ -114,7 +121,7 @@ Q_DECL_EXPORT jint JNICALL JNI_OnLoad(JavaVM *vm, void*)
         qCWarning(Log) << "Failed to get JNI environment.";
         return -1;
     }
-    jclass cls = env->FindClass("org/kde/itinerary/Activity");
+    jclass cls = env->FindClass(KAndroidExtras::Jni::typeName<ItineraryActivity>());
     if (env->RegisterNatives(cls, methods, sizeof(methods) / sizeof(JNINativeMethod)) < 0) {
         qCWarning(Log) << "Failed to register native functions.";
         return -1;
@@ -144,6 +151,11 @@ ApplicationController* ApplicationController::instance()
     return s_instance;
 }
 
+void ApplicationController::requestOpenPage(const QString &page)
+{
+    Q_EMIT openPageRequested(page);
+}
+
 void ApplicationController::setReservationManager(ReservationManager* resMgr)
 {
     m_resMgr = resMgr;
@@ -152,6 +164,8 @@ void ApplicationController::setReservationManager(ReservationManager* resMgr)
 void ApplicationController::setPkPassManager(PkPassManager* pkPassMgr)
 {
     m_pkPassMgr = pkPassMgr;
+    connect(m_pkPassMgr, &PkPassManager::passAdded, this, &ApplicationController::importPass);
+    connect(m_pkPassMgr, &PkPassManager::passUpdated, this, &ApplicationController::importPass);
 }
 
 void ApplicationController::setDocumentManager(DocumentManager* docMgr)
@@ -195,20 +209,26 @@ void ApplicationController::importFromIntent(const KAndroidExtras::Intent &inten
         return;
     }
 
-    // opening a file
+    // opening a URL, can be something to import or a shortcut path
     if (action == Intent::ACTION_VIEW) {
-        importFromUrl(intent.getData());
+        const QUrl url = intent.getData();
+        if (url.scheme() == QLatin1String("page")) {
+            qCDebug(Log) << url;
+            requestOpenPage(url.path().mid(1));
+        } else {
+            importFromUrl(intent.getData());
+        }
         return;
     }
 
     // shared data, e.g. from email applications like FairMail
     if (action == Intent::ACTION_SEND || action == Intent::ACTION_SEND_MULTIPLE) {
-        const auto type = intent.getType();
+        const QString type = intent.getType();
         const auto subject = intent.getStringExtra(Intent::EXTRA_SUBJECT);
         const auto from = intent.getStringArrayExtra(Intent::EXTRA_EMAIL);
         const auto text = intent.getStringExtra(Intent::EXTRA_TEXT);
         qCInfo(Log) << action << type << subject << from << text;
-        const auto attachments = Jni::fromArray<QStringList>(QtAndroid::androidActivity().callObjectMethod("attachmentsForIntent", Jni::signature<Jni::Array<java::lang::String>(android::content::Intent)>(), static_cast<QAndroidJniObject>(intent).object()));
+        const QStringList attachments = ItineraryActivity().attachmentsForIntent(intent);
         qCInfo(Log) << attachments;
 
         KMime::Message msg;
@@ -246,7 +266,7 @@ void ApplicationController::importFromIntent(const KAndroidExtras::Intent &inten
 
         msg.assemble();
         qDebug().noquote() << msg.encodedContent();
-        m_resMgr->importReservation(&msg);
+        importMimeMessage(&msg);
         return;
     }
 
@@ -281,7 +301,7 @@ void ApplicationController::importFromUrl(const QUrl &url)
     }
 
     qCDebug(Log) << url;
-    if (url.isLocalFile() || url.scheme() == QLatin1String("content")) {
+    if (FileHelper::isLocalFile(url)) {
         importLocalFile(url);
         return;
     }
@@ -298,6 +318,7 @@ void ApplicationController::importFromUrl(const QUrl &url)
         connect(reply, &QNetworkReply::finished, this, [this, reply]() {
             if (reply->error() != QNetworkReply::NoError) {
                 qCDebug(Log) << reply->url() << reply->errorString();
+                Q_EMIT infoMessage(i18n("Download failed: %1", reply->errorString()));
                 return;
             }
             importData(reply->readAll());
@@ -315,67 +336,90 @@ void ApplicationController::importLocalFile(const QUrl &url)
         return;
     }
 
-    QFile f(url.isLocalFile() ? url.toLocalFile() : url.toString());
+    QFile f(FileHelper::toLocalFile(url));
     if (!f.open(QFile::ReadOnly)) {
         qCWarning(Log) << "Failed to open" << f.fileName() << f.errorString();
+        Q_EMIT infoMessage(i18n("Import failed: %1", f.errorString()));
         return;
     }
     if (f.size() > 4000000 && !f.fileName().endsWith(QLatin1String(".itinerary"))) {
         qCWarning(Log) << "File too large, ignoring" << f.fileName() << f.size();
+        Q_EMIT infoMessage(i18n("Import failed: File too large."));
         return;
     }
 
+    // deal with things we can import more efficiently from a file directly
     const auto head = f.peek(4);
-    if (url.fileName().endsWith(QLatin1String(".pkpass"), Qt::CaseInsensitive)) {
-        m_pkPassMgr->importPass(url);
-    } else if (url.fileName().endsWith(QLatin1String(".itinerary"), Qt::CaseInsensitive)) {
-        importBundle(url);
-    } else if (strncmp(head.constData(), "PK\x03\x04", 4) == 0) {
-        if (m_pkPassMgr->importPass(url).isEmpty()) {
-            importBundle(url);
-        }
-    } else {
-        const auto data = f.readAll();
-        const auto resIds = importReservationOrHealthCertificate(data, f.fileName());
-        if (resIds.empty()) {
+    if (FileHelper::hasZipHeader(head)) {
+        if (url.fileName().endsWith(QLatin1String(".pkpass"), Qt::CaseInsensitive) && !m_pkPassMgr->importPass(url).isEmpty()) {
             return;
         }
-
-        // check if there is a document we want to attach here
-        QMimeDatabase db;
-        const auto mt = db.mimeTypeForFileNameAndData(f.fileName(), data);
-        if (mt.name() != QLatin1String("application/pdf")) { // TODO support more file types (however we certainly want to exclude pkpass and json here)
+        if (url.fileName().endsWith(QLatin1String(".itinerary"), Qt::CaseInsensitive) && importBundle(url)) {
             return;
         }
-
-        DigitalDocument docInfo;
-        docInfo.setName(f.fileName());
-        docInfo.setEncodingFormat(mt.name());
-        const auto docId = DocumentUtil::idForContent(data);
-        m_docMgr->addDocument(docId, docInfo, data);
-
-        for (const auto &resId : resIds) {
-            auto res = m_resMgr->reservation(resId);
-            if (DocumentUtil::addDocumentId(res, docId)) {
-                m_resMgr->updateReservation(resId, res);
-            }
-        }
     }
+
+    importData(f.readAll(), f.fileName());
 }
 
-void ApplicationController::importData(const QByteArray &data)
+void ApplicationController::importData(const QByteArray &data, const QString &fileName)
 {
     qCDebug(Log);
     if (data.size() < 4) {
         return;
     }
-    if (strncmp(data.constData(), "PK\x03\x04", 4) == 0) {
-        if (m_pkPassMgr->importPassFromData(data).isEmpty()) {
-            importBundle(data);
+
+    if (FileHelper::hasZipHeader(data)) {
+        if (!m_pkPassMgr->importPassFromData(data).isEmpty()) {
+            return;
+        }
+        if (importBundle(data)) {
+            return;
         }
-    } else {
-        importReservationOrHealthCertificate(data);
     }
+
+    using namespace KItinerary;
+    ExtractorEngine engine;
+#if KITINERARY_VERSION >= QT_VERSION_CHECK(5, 19, 41)
+    // user opened the file, so we can be reasonably sure they assume it contains
+    // relevant content, so try expensive extraction methods too
+    engine.setHints(ExtractorEngine::ExtractFullPageRasterImages);
+#endif
+    engine.setContextDate(QDateTime(QDate::currentDate(), QTime(0, 0)));
+    engine.setData(data, fileName);
+    const auto resIds = m_resMgr->importReservations(JsonLdDocument::fromJson(engine.extract()));
+    if (!resIds.isEmpty()) {
+        // check if there is a document we want to attach here
+        QMimeDatabase db;
+        const auto mt = db.mimeTypeForFileNameAndData(fileName, data);
+        if (mt.name() == QLatin1String("application/pdf")) { // TODO support more file types (however we certainly want to exclude pkpass and json here)
+            DigitalDocument docInfo;
+            docInfo.setName(fileName);
+            docInfo.setEncodingFormat(mt.name());
+            const auto docId = DocumentUtil::idForContent(data);
+            m_docMgr->addDocument(docId, docInfo, data);
+
+            for (const auto &resId : resIds) {
+                auto res = m_resMgr->reservation(resId);
+                if (DocumentUtil::addDocumentId(res, docId)) {
+                    m_resMgr->updateReservation(resId, res);
+                }
+            }
+        }
+
+        Q_EMIT infoMessage(i18np("One reservation imported.", "%1 reservations imported.", resIds.size()));
+        return;
+    }
+
+    // look for health certificate barcodes instead
+    // if we don't find anything, try to import as health certificate directly
+    if (importHealthCertificateRecursive(engine.rootDocumentNode()) || m_healthCertMgr->importCertificate(data)) {
+        Q_EMIT infoMessage(i18n("Health certificate imported."));
+        return;
+    }
+
+    // nothing found
+    Q_EMIT infoMessage(i18n("Nothing imported."));
 }
 
 void ApplicationController::checkCalendar()
@@ -404,17 +448,17 @@ bool ApplicationController::hasClipboardContent() const
 }
 
 
-void ApplicationController::exportToFile(const QString &filePath)
+void ApplicationController::exportToFile(const QUrl &url)
 {
-    qCDebug(Log) << filePath;
-    if (filePath.isEmpty()) {
+    if (url.isEmpty()) {
         return;
     }
 
-    File f(QUrl(filePath).isLocalFile() ? QUrl(filePath).toLocalFile() : filePath);
+    qCDebug(Log) << url;
+    File f(FileHelper::toLocalFile(url));
     if (!f.open(File::Write)) {
         qCWarning(Log) << f.errorString();
-        // TODO show error in UI
+        Q_EMIT infoMessage(i18n("Export failed: %1", f.errorString()));
         return;
     }
 
@@ -427,17 +471,19 @@ void ApplicationController::exportToFile(const QString &filePath)
     exporter.exportHealthCertificates(m_healthCertMgr);
     exporter.exportLiveData();
     exporter.exportSettings();
+    Q_EMIT infoMessage(i18n("Export completed."));
 }
 
-void ApplicationController::exportTripToGpx(const QString &tripGroupId, const QString &filePath)
+void ApplicationController::exportTripToGpx(const QString &tripGroupId, const QUrl &url)
 {
-    if (filePath.isEmpty()) {
+    if (url.isEmpty()) {
         return;
     }
 
-    QFile f(QUrl(filePath).isLocalFile() ? QUrl(filePath).toLocalFile() : filePath);
+    QFile f(FileHelper::toLocalFile(url));
     if (!f.open(QFile::WriteOnly)) {
         qCWarning(Log) << f.errorString() << f.fileName();
+        Q_EMIT infoMessage(i18n("Export failed: %1", f.errorString()));
         return;
     }
     GpxExport exporter(&f);
@@ -450,72 +496,59 @@ void ApplicationController::exportTripToGpx(const QString &tripGroupId, const QS
         const auto transferAfter = m_transferMgr->transfer(batchId, Transfer::After);
         exporter.writeReservation(res, m_liveDataMgr->journey(batchId), transferBefore, transferAfter);
     }
+    Q_EMIT infoMessage(i18n("Export completed."));
 }
 
-void ApplicationController::importBundle(const QUrl &url)
+bool ApplicationController::importBundle(const QUrl &url)
 {
-    KItinerary::File f(url.isLocalFile() ? url.toLocalFile() : url.toString());
+    KItinerary::File f(FileHelper::toLocalFile(url));
     if (!f.open(File::Read)) {
-        // TODO show error in the ui
         qCWarning(Log) << "Failed to open bundle file:" << url << f.errorString();
-        return;
+        Q_EMIT infoMessage(i18n("Import failed: %1", f.errorString()));
+        return false;
     }
 
-    importBundle(&f);
+    return importBundle(&f);
 }
 
-void ApplicationController::importBundle(const QByteArray &data)
+bool ApplicationController::importBundle(const QByteArray &data)
 {
     QBuffer buffer;
     buffer.setData(data);
     buffer.open(QBuffer::ReadOnly);
     KItinerary::File f(&buffer);
     if (!f.open(File::Read)) {
-        // TODO show error in the ui
         qCWarning(Log) << "Failed to open bundle data:" << f.errorString();
-        return;
+        Q_EMIT infoMessage(i18n("Import failed: %1", f.errorString()));
+        return false;
     }
 
-    importBundle(&f);
+    return importBundle(&f);
 }
 
-void ApplicationController::importBundle(KItinerary::File *file)
+bool ApplicationController::importBundle(KItinerary::File *file)
 {
     Importer importer(file);
-    importer.importReservations(m_resMgr);
-    importer.importPasses(m_pkPassMgr);
-    importer.importDocuments(m_docMgr);
-    importer.importFavoriteLocations(m_favLocModel);
-    importer.importTransfers(m_resMgr, m_transferMgr);
-    importer.importHealthCertificates(m_healthCertMgr);
-    importer.importLiveData(m_liveDataMgr);
-    importer.importSettings();
-
-    // favorite locations
-    auto favLocs = FavoriteLocation::fromJson(QJsonDocument::fromJson(file->customData(QStringLiteral("org.kde.itinerary/favorite-locations"), QStringLiteral("locations"))).array());
-    if (!favLocs.empty()) {
-        m_favLocModel->setFavoriteLocations(std::move(favLocs));
-    }
-}
-
-QVector<QString> ApplicationController::importReservationOrHealthCertificate(const QByteArray &data, const QString &fileName)
-{
-    using namespace KItinerary;
-    ExtractorEngine engine;
-    engine.setContextDate(QDateTime(QDate::currentDate(), QTime(0, 0)));
-    engine.setData(data, fileName);
-    const auto resIds = m_resMgr->importReservations(JsonLdDocument::fromJson(engine.extract()));
-    if (!resIds.isEmpty()) {
-        return resIds;
+    int count = 0;
+    {
+        QSignalBlocker blocker(this); // suppress infoMessage()
+        count += importer.importReservations(m_resMgr);
+        count += importer.importPasses(m_pkPassMgr);
+        count += importer.importDocuments(m_docMgr);
+        count += importer.importFavoriteLocations(m_favLocModel);
+        count += importer.importTransfers(m_resMgr, m_transferMgr);
+        count += importer.importHealthCertificates(m_healthCertMgr);
+        count += importer.importLiveData(m_liveDataMgr);
+        count += importer.importSettings();
     }
 
-    // look for health certificate barcodes instead
-    importHealthCertificateRecursive(engine.rootDocumentNode());
-
-    return {};
+    if (count > 0) {
+        Q_EMIT infoMessage(i18n("Import completed."));
+    }
+    return count > 0;
 }
 
-void ApplicationController::importHealthCertificateRecursive(const ExtractorDocumentNode &node)
+bool ApplicationController::importHealthCertificateRecursive(const ExtractorDocumentNode &node)
 {
     if (node.childNodes().size() == 1 && node.mimeType() == QLatin1String("internal/qimage")) {
         const auto &child = node.childNodes()[0];
@@ -527,8 +560,33 @@ void ApplicationController::importHealthCertificateRecursive(const ExtractorDocu
         }
     }
 
+    bool result = false;
     for (const auto &child : node.childNodes()) {
-        importHealthCertificateRecursive(child);
+        if (importHealthCertificateRecursive(child)) { // no shortcut evaluation, more than one QR code per PDF is possible
+            result = true;
+        }
+    }
+    return result;
+}
+
+void ApplicationController::importPass(const QString &passId)
+{
+    const auto pass = m_pkPassMgr->pass(passId);
+    KItinerary::ExtractorEngine engine;
+    engine.setContent(QVariant::fromValue<KPkPass::Pass*>(pass), u"application/vnd.apple.pkpass");
+    const auto resIds = m_resMgr->importReservations(JsonLdDocument::fromJson(engine.extract()));
+    if (!resIds.isEmpty()) {
+        Q_EMIT infoMessage(i18np("One reservation imported.", "%1 reservations imported.", resIds.size()));
+    }
+}
+
+void ApplicationController::importMimeMessage(KMime::Message *msg)
+{
+    ExtractorEngine engine;
+    engine.setContent(QVariant::fromValue<KMime::Content*>(msg), u"message/rfc822");
+    const auto resIds = m_resMgr->importReservations(JsonLdDocument::fromJson(engine.extract()));
+    if (!resIds.isEmpty()) {
+        Q_EMIT infoMessage(i18np("One reservation imported.", "%1 reservations imported.", resIds.size()));
     }
 }
 
@@ -548,11 +606,11 @@ void ApplicationController::addDocument(const QString &batchId, const QUrl &url)
     docInfo.setName(KAndroidExtras::ContentResolver::fileName(url));
 #else
     QMimeDatabase db;
-    docInfo.setEncodingFormat(db.mimeTypeForFile(url.isLocalFile() ? url.toLocalFile() : url.toString()).name());
+    docInfo.setEncodingFormat(db.mimeTypeForFile(FileHelper::toLocalFile(url)).name());
     docInfo.setName(url.fileName());
 #endif
 
-    m_docMgr->addDocument(docId, docInfo, url.isLocalFile() ? url.toLocalFile() : url.toString());
+    m_docMgr->addDocument(docId, docInfo, FileHelper::toLocalFile(url));
 
     const auto resIds = m_resMgr->reservationsForBatch(batchId);
     for (const auto &resId : resIds) {
@@ -577,7 +635,7 @@ void ApplicationController::openDocument(const QUrl &url)
 #ifdef Q_OS_ANDROID
     using namespace KAndroidExtras;
     auto activity = QtAndroid::androidActivity();
-    auto uri = activity.callObjectMethod("openDocument", Jni::signature<android::net::Uri(java::lang::String)>(), QAndroidJniObject::fromString(url.toLocalFile()).object());
+    auto uri = ItineraryActivity().openDocument(url.toLocalFile());
 
     Intent intent;
     intent.setData(uri);
@@ -598,3 +656,10 @@ QString ApplicationController::extractorCapabilities() const
 {
     return ExtractorCapabilities::capabilitiesString();
 }
+
+QVariant ApplicationController::aboutData() const
+{
+    return QVariant::fromValue(KAboutData::applicationData());
+}
+
+#include "moc_applicationcontroller.cpp"
