@@ -7,6 +7,8 @@
 // list all headers here that have no .cpp file
 // this only serves as a guarantee that these headers actually compile
 #include "jniarray.h"
+#include "jnicommon.h"
+#include "jnimethod.h"
 #include "jniproperty.h"
 #include "jnisignature.h"
 #include "jnitypes.h"
